@@ -57,8 +57,8 @@ class OpenAIProvider(ILLMClient):
         self._pending_client_hooks: list[tuple[str, Callable[[dict[str, Any]], None]]] = []
         if self._client is not None:
             self._ensure_client_hooks()
-        # Pending outputs buffer for streaming follow-ups
-        self._pending_outputs: list[dict] = []
+        # Pending outputs buffer (keyed by previous_response_id)
+        self._pending_outputs: dict[str, list[dict]] = {}
 
     PROVIDER_EVENTS = {
         "before_send",
@@ -518,9 +518,24 @@ class OpenAIProvider(ILLMClient):
                 setattr(self._client, "_forgebase_include_output_name", include_output_name)  # type: ignore[attr-defined]
             except Exception:
                 pass
+            def _collect_call_ids_from_event(ev: dict) -> list[str]:
+                ids: list[str] = []
+                def walk(obj: Any) -> None:  # type: ignore[override]
+                    if isinstance(obj, dict):
+                        for k, v in obj.items():
+                            if k in {"id", "call_id"} and isinstance(v, str) and v.startswith("call_"):
+                                ids.append(v)
+                            walk(v)
+                    elif isinstance(obj, list):
+                        for it in obj:
+                            walk(it)
+                walk(ev)
+                return ids
+
             for _ in range(3):
                 any_delta = False
-                sent_pending_this_call = False
+                collected_call_ids: list[str] = []
+                current_prev = previous_response_id
                 events = client.stream_events(
                     prompt,
                     tools=self._tools_defs or None,
@@ -533,6 +548,13 @@ class OpenAIProvider(ILLMClient):
                 response_obj: Optional[ResponseResult] = None
                 for event in events:
                     event_type = event.get("type")
+                    # Track any call_* ids surfaced in streaming events
+                    try:
+                        for cid in _collect_call_ids_from_event(event):
+                            if cid not in collected_call_ids:
+                                collected_call_ids.append(cid)
+                    except Exception:
+                        pass
                     if event_type == "response.output_text.delta":
                         delta = event.get("delta", "")
                         if delta:
@@ -551,10 +573,43 @@ class OpenAIProvider(ILLMClient):
                     break
 
                 previous_response_id = response_obj.id
-                # If we successfully completed a call, clear pending outputs that were sent
-                if self._pending_outputs and sent_pending_this_call:
-                    self._pending_outputs.clear()
+                # Clear pending outputs for the previous follow-up if any
+                if current_prev and current_prev in self._pending_outputs:
+                    self._pending_outputs.pop(current_prev, None)
                 uses = self._collect_tool_uses(response_obj)
+                # Supplement call_* ids from input_items if needed
+                try:
+                    extra_ids: list[str] = []
+                    if hasattr(client, "response") and isinstance(getattr(client, "response", None), OpenAIResponse):
+                        items = client.response.list_input_items(response_obj.id)  # type: ignore[attr-defined]
+                        seq = []
+                        if isinstance(items, dict) and "data" in items:
+                            seq = items.get("data") or []
+                        elif isinstance(items, list):
+                            seq = items
+                        for it in seq or []:
+                            t = (it.get("type") if isinstance(it, dict) else None) or ""
+                            cid = (it.get("call_id") if isinstance(it, dict) else None) or ""
+                            if t == "function_call" and isinstance(cid, str) and cid.startswith("call_"):
+                                if cid not in collected_call_ids and cid not in extra_ids:
+                                    extra_ids.append(cid)
+                    collected_call_ids.extend(extra_ids)
+                except Exception:
+                    pass
+                # Reconcile function call ids: prefer unique call_* ids in order
+                call_iter = iter(collected_call_ids)
+                for u in uses:
+                    if u.get("kind") == "function":
+                        uid = str(u.get("id", ""))
+                        if not uid.startswith("call_"):
+                            try:
+                                new_id = next(call_iter)
+                                u["id"] = new_id
+                                raw = u.get("raw")
+                                if isinstance(raw, dict):
+                                    raw["call_id"] = new_id
+                            except StopIteration:
+                                pass
                 if not uses:
                     if not any_delta:
                         fallback = _extract_text(response_obj)
@@ -578,18 +633,18 @@ class OpenAIProvider(ILLMClient):
                                 next_context.append(raw)
                             elif not is_real:
                                 next_context.append(raw)
-                # Merge pending outputs with new outputs to ensure none is lost
+                # Merge pending outputs for this response id with new outputs
+                pending_for_resp = list(self._pending_outputs.get(response_obj.id, []))
                 merged_outputs: list[dict] = []
-                if self._pending_outputs:
-                    merged_outputs.extend(self._pending_outputs)
+                if pending_for_resp:
+                    merged_outputs.extend(pending_for_resp)
                 merged_outputs.extend(tool_results)
                 next_context.extend(merged_outputs)
-                # Mark that we will send pending outputs on next call
+                # Store pending for this response id (will be cleared on next completed)
                 if merged_outputs:
-                    self._pending_outputs = merged_outputs
-                    sent_pending_this_call = True
+                    self._pending_outputs[response_obj.id] = merged_outputs
                 else:
-                    sent_pending_this_call = False
+                    self._pending_outputs.pop(response_obj.id, None)
                 context_items = next_context
 
                 # Debug logging of follow-up payload (ids/types only)
@@ -601,7 +656,8 @@ class OpenAIProvider(ILLMClient):
                         }]
                         types = [it.get("type") for it in (context_items or []) if isinstance(it, dict)]
                         cids = [it.get('call_id') for it in outs]
-                        msg = f"[follow-up] prev={previous_response_id} outputs={len(outs)} types={types} call_ids={cids}"
+                        unique_c = len(set([cid for cid in cids if cid]))
+                        msg = f"[follow-up] prev={previous_response_id} outputs={len(outs)} types={types} call_ids={cids} unique={unique_c}"
                         # Log também em INFO para garantir visibilidade em produção
                         _log.info(msg)
                     except Exception:
